#include <libfilezilla/format.hpp>

#include "util.hpp"

#include "../strsyserror.hpp"

namespace fz::tvfs {

std::string to_string(result r)
{
	using namespace std::string_literals;

	switch (r.error_) {
		case result::noperm:         return "Permission denied"s;
		case result::nofile:         return "Couldn't open the file"s;
		case result::nodir:          return "Couldn't open the directory"s;
		case result::other: {
			constexpr decltype(r.raw_) raw =
	#ifdef FZ_WINDOWS
				ERROR_CALL_NOT_IMPLEMENTED;
	#else
				ENOSYS;
	#endif

			if (r.raw_ == raw) {
				return "Operation not implemented"s;
			}

			if (r.raw_) {
				return to_utf8(strsyserror(raw));
			}

			return "Couldn't open the file or directory"s;
		}

		case result::ok:             return "No error"s;
		case result::nospace:        return "No space left"s;
		case result::invalid:        return "Invalid file name or path"s;
		case result::resource_limit: return "Too many open files or directories"s;
	}

	return {};
}

}
