#include "../administrator.hpp"
#include "../../filezilla/util/io.hpp"

auto administrator::operator()(administration::generate_selfsigned_certificate &&v)
{
	auto [id, distinguished_name, hostnames] = std::move(v).tuple();

	auto cert = fz::securable_socket::cert_info::generate_selfsigned(config_paths_.certificates(), &logger_, {}, distinguished_name, hostnames);

	if (cert)
		return v.success(id, std::move(cert), cert.load_extra(&logger_));

	// FIXME: Ideally we should return a proper failure() here, but then there'd be no way to distinguish between ids.
	return v.success(id, fz::securable_socket::cert_info{}, fz::securable_socket::cert_info::extra{});
}

auto administrator::operator()(administration::upload_certificate &&v)
{
	auto [id, cert, key, password] = std::move(v).tuple();

	auto info = fz::securable_socket::cert_info::import_certificate_files(config_paths_.certificates(), cert, key, password, &logger_);

	if (info)
		return v.success(id, std::move(info), info.load_extra(&logger_));

	// FIXME: Ideally we should return a proper failure() here, but then there'd be no way to distinguish between ids.
	return v.success(id, fz::securable_socket::cert_info{}, fz::securable_socket::cert_info::extra{});
}

auto administrator::operator()(administration::get_extra_certs_info &&v)
{
	auto && [id, info] = std::move(v).tuple();

	struct log_to_string: fz::logger_interface
	{
		log_to_string(fz::native_string &str)
			: str_(str)
		{
			level_ = fz::logmsg::error;
		}

		void do_log(fz::logmsg::type, std::wstring && msg) override
		{
			if (!str_.empty())
				str_.append(fzT("\n"));

			str_.append(fz::to_native(msg));
		}

	private:
		fz::native_string &str_;
	};

	fz::native_string error;
	log_to_string logger(error);
	fz::securable_socket::cert_info::extra extra;

	if (info.set_root_path(config_paths_.certificates(), &logger)) {
		if (auto key = fz::util::io::read(info.key_path()); key.empty()) {
			error = fzT("Couldn't read key file.");
		}
		else
		if (auto certs = fz::util::io::read(info.certs_path()); certs.empty()) {
			error = fzT("Couldn't read certificates file.");
		}
		else {
			error = fz::check_certificate_status(key.to_view(), certs.to_view(), info.password());
		}

		if (error.empty()) {
			extra = info.load_extra(&logger);
		}
	}

	return v.success(id, std::move(info), std::move(extra), std::move(error));
}


FZ_RMP_INSTANTIATE_HERE_DISPATCHING_FOR(administration::engine, administrator, administration::generate_selfsigned_certificate);
FZ_RMP_INSTANTIATE_HERE_DISPATCHING_FOR(administration::engine, administrator, administration::upload_certificate);
FZ_RMP_INSTANTIATE_HERE_DISPATCHING_FOR(administration::engine, administrator, administration::get_extra_certs_info);
